@@ -26,9 +26,9 @@ class Command(BaseCommand):
             default=DEFAULT_SHAPEFILES_DIR,
             help='Load shapefiles from this directory'),
         make_option('-e', '--except', action='store', dest='except',
-            help='Don\'t load these kinds of Areas, comma-delimitted.'),
+            help='Don\'t load these kinds of Areas, comma-delimited.'),
         make_option('-o', '--only', action='store', dest='only',
-            help='Only load these kinds of Areas, comma-delimitted.'),
+            help='Only load these kinds of Areas, comma-delimited.'),
     )
 
     def get_version(self):
@@ -41,7 +41,7 @@ class Command(BaseCommand):
 
         if options['only']:
             only = options['only'].split(',')
-            # TODO: stripping whitespace here because optparse doesn't handle it correclty
+            # TODO: stripping whitespace here because optparse doesn't handle it correctly
             sources = [s for s in SHAPEFILES if s.replace(' ', '') in only]
         elif options['except']:
             exceptions = options['except'].upper().split(',')
@@ -50,11 +50,6 @@ class Command(BaseCommand):
         else:
             sources = [s for s in SHAPEFILES]
         
-        # Get spatial reference system for the postgis geometry field
-        geometry_field = Boundary._meta.get_field_by_name(GEOMETRY_COLUMN)[0]
-        SpatialRefSys = connections[DEFAULT_DB_ALIAS].ops.spatial_ref_sys()
-        db_srs = SpatialRefSys.objects.get(srid=geometry_field.srid).srs
-
         for kind, config in SHAPEFILES.items():
             if kind not in sources:
                 log.info('Skipping %s.' % kind)
@@ -77,20 +72,9 @@ class Command(BaseCommand):
                     log.info('Loading new %s.' % kind)
 
             path = os.path.join(options['data_dir'], config['file'])
-            datasource = create_datasource(path)
+            datasources = create_datasources(path)
 
-            # Assume only a single-layer in shapefile
-            if datasource.layer_count > 1:
-                log.warn('%s shapefile has multiple layers, using first.' % kind)
-
-            layer = datasource[0]
-            if 'srid' in config and config['srid']:
-                layer_srs = SpatialRefSys.objects.get(srid=config['srid']).srs
-            else:
-                layer_srs = layer.srs
-
-            # Create a convertor to turn the source data into
-            transformer = CoordTransform(layer_srs, db_srs)
+            layer = datasources[0][0]
 
             # Create BoundarySet
             set = BoundarySet.objects.create(
@@ -105,59 +89,16 @@ class Command(BaseCommand):
                 count=len(layer),
                 metadata_fields=layer.fields)
 
-            for feature in layer:
-                # Transform the geometry to the correct SRS
-                geometry = self.polygon_to_multipolygon(feature.geom)
-                geometry.transform(transformer)
-    
-                # Create simplified geometry field by collapsing points within 1/1000th of a degree.
-                # Since Chicago is at approx. 42 degrees latitude this works out to an margin of 
-                # roughly 80 meters latitude and 112 meters longitude.
-                # Preserve topology prevents a shape from ever crossing over itself.
-                simple_geometry = geometry.geos.simplify(0.0001, preserve_topology=True)
-                
-                # Conversion may force multipolygons back to being polygons
-                simple_geometry = self.polygon_to_multipolygon(simple_geometry.ogr)
-
-                # Extract metadata into a dictionary
-                metadata = {}
-
-                for field in layer.fields:
-                    
-                    # Decode string fields using encoding specified in definitions config
-                    if config['encoding'] != '':
-                        try:
-                            metadata[field] = feature.get(field).decode(config['encoding'])
-                        # Only strings will be decoded, get value in normal way if int etc.
-                        except AttributeError:
-                            metadata[field] = feature.get(field)
-                    else:
-                        metadata[field] = feature.get(field)
-
-                external_id = config['ider'](feature)
-                feature_name = config['namer'](feature)
-                
-                # If encoding is specified, decode id and feature name
-                if config['encoding'] != '':
-                    external_id = external_id.decode(config['encoding'])
-                    feature_name = feature_name.decode(config['encoding'])
-
-                if config['kind_first']:
-                    display_name = '%s %s' % (config['singular'], feature_name)
-                else:
-                    display_name = '%s %s' % (feature_name, config['singular'])
-
-                Boundary.objects.create(
-                    set=set,
-                    kind=config['singular'],
-                    external_id=external_id,
-                    name=feature_name,
-                    display_name=display_name,
-                    metadata=metadata,
-                    shape=geometry.wkt,
-                    simple_shape=simple_geometry.wkt,
-                    centroid=geometry.geos.centroid)
-
+            for datasource in datasources:
+                log.info("Loading %s from %s" % (kind, datasource.name))
+                # Assume only a single-layer in shapefile
+                if datasource.layer_count > 1:
+                    log.warn('%s shapefile [%s] has multiple layers, using first.' % (datasource.name, kind))
+                self.add_boundaries_for_layer(config,layer,set)
+            
+            set.count = Boundary.objects.filter(set=set).count() # sync this with reality
+            set.save()
+            # TODO: work through additional shapefiles, increment set.count
             log.info('Saved %i %s.' % (set.count, kind))
 
     def polygon_to_multipolygon(self, geom):
@@ -173,11 +114,91 @@ class Command(BaseCommand):
         else:
             raise ValueError('Geom is neither Polygon nor MultiPolygon.')
 
-def create_datasource(path):
+
+    def add_boundaries_for_layer(self,config,layer,set):
+        # Get spatial reference system for the postgis geometry field
+        geometry_field = Boundary._meta.get_field_by_name(GEOMETRY_COLUMN)[0]
+        SpatialRefSys = connections[DEFAULT_DB_ALIAS].ops.spatial_ref_sys()
+        db_srs = SpatialRefSys.objects.get(srid=geometry_field.srid).srs
+
+
+        if 'srid' in config and config['srid']:
+            layer_srs = SpatialRefSys.objects.get(srid=config['srid']).srs
+        else:
+            layer_srs = layer.srs
+
+        # Create a convertor to turn the source data into
+        transformer = CoordTransform(layer_srs, db_srs)
+
+        for feature in layer:
+            # Transform the geometry to the correct SRS
+            geometry = self.polygon_to_multipolygon(feature.geom)
+            geometry.transform(transformer)
+
+            # Create simplified geometry field by collapsing points within 1/1000th of a degree.
+            # Since Chicago is at approx. 42 degrees latitude this works out to an margin of 
+            # roughly 80 meters latitude and 112 meters longitude.
+            # Preserve topology prevents a shape from ever crossing over itself.
+            simple_geometry = geometry.geos.simplify(0.0001, preserve_topology=True)
+            
+            # Conversion may force multipolygons back to being polygons
+            simple_geometry = self.polygon_to_multipolygon(simple_geometry.ogr)
+
+            # Extract metadata into a dictionary
+            metadata = {}
+
+            for field in layer.fields:
+                
+                # Decode string fields using encoding specified in definitions config
+                if config['encoding'] != '':
+                    try:
+                        metadata[field] = feature.get(field).decode(config['encoding'])
+                    # Only strings will be decoded, get value in normal way if int etc.
+                    except AttributeError:
+                        metadata[field] = feature.get(field)
+                else:
+                    metadata[field] = feature.get(field)
+
+            external_id = config['ider'](feature)
+            feature_name = config['namer'](feature)
+            
+            # If encoding is specified, decode id and feature name
+            if config['encoding'] != '':
+                external_id = external_id.decode(config['encoding'])
+                feature_name = feature_name.decode(config['encoding'])
+
+            if config['kind_first']:
+                display_name = '%s %s' % (config['singular'], feature_name)
+            else:
+                display_name = '%s %s' % (feature_name, config['singular'])
+
+            Boundary.objects.create(
+                set=set,
+                kind=config['singular'],
+                external_id=external_id,
+                name=feature_name,
+                display_name=display_name,
+                metadata=metadata,
+                shape=geometry.wkt,
+                simple_shape=simple_geometry.wkt,
+                centroid=geometry.geos.centroid)
+        
+def create_datasources(path):
     if path.endswith('.zip'):
         path = temp_shapefile_from_zip(path)
+
+    if path.endswith('.shp'):
+        return [DataSource(path)]
     
-    return DataSource(path)
+    # assume it's a directory...
+    sources = []
+    for fn in os.listdir(path):
+        fn = os.path.join(path,fn)
+        if fn.endswith('.zip'):
+            fn = temp_shapefile_from_zip(fn)
+        if fn.endswith('.shp'):
+            sources.append(DataSource(fn))
+    return sources
     
 def temp_shapefile_from_zip(zip_path):
     """Given a path to a ZIP file, unpack it into a temp dir and return the path
